from __future__ import unicode_literals

from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import pgettext
from django.utils.encoding import python_2_unicode_compatible
from django_extensions.db.models import TimeStampedModel
from django.utils import timezone
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType

from evotemailings.mixins import MailingListMixin
from evotelogs.decorators import auditable

from .managers import MessageQueryset, MessageContactManager
from .tasks import send_messages


class MessageContact(MailingListMixin, TimeStampedModel):
    """ """
    user = models.ForeignKey(settings.AUTH_USER_MODEL, null=True)
    namespace = models.CharField(max_length=64, db_index=True)
    name = models.CharField(_('name'), max_length=128, null=True)
    email = models.EmailField(_('email'), max_length=128, null=True)
    objects = MessageContactManager()

    class Meta:
        verbose_name = _("message contact")
        verbose_name_plural = _("message contacts")
        index_together = ('user', 'namespace')

    def __str__(self):
        return "{} ({})".format(self.get_name(), self.get_email())

    def get_name(self):
        if not self.name and self.user_id:
            return self.user.name
        return self.name

    def get_email(self):
        if not self.email and self.user_id:
            return self.user.get_email()
        return self.email

    @property
    def is_anonymous(self):
        return bool(self.user_id is None)

    def get_mailing_list(self):
        return [(self.get_name(), self.get_email())]


@auditable
@python_2_unicode_compatible
class Message(TimeStampedModel):

    class Folder:
        RECEIVED = 0
        SENT = 1
        ARCHIVED = 2

        choices = (
            (RECEIVED, _("Received")),
            (SENT, _("Sent")),
            (ARCHIVED, _("Archived")),
        )

    class State:
        NEW = 0
        READ = 1
        REPLIED = 2

        choices = (
            (NEW, pgettext("message", "New")),
            (READ, _("Read")),
            (REPLIED, _("Replied")),
        )

    sender = models.ForeignKey(
        MessageContact, verbose_name=_("sender"), related_name='sent')
    recipient = models.ForeignKey(
        MessageContact, verbose_name=_("recipient"), null=True)
    subject = models.CharField(_("subject"), max_length=260)
    body = models.TextField(_("body"))
    read_at = models.DateTimeField(_("read at"), null=True, blank=True)
    parent = models.ForeignKey(
        'self', verbose_name=_("parent message"), null=True,
        related_name='siblings')
    folder = models.PositiveSmallIntegerField(
        _("folder"), choices=Folder.choices, default=Folder.SENT)
    state = models.PositiveSmallIntegerField(
        _("state"), choices=State.choices, default=State.NEW)
    content_type = models.ForeignKey(ContentType, null=True)
    object_id = models.PositiveIntegerField(null=True)
    content_object = GenericForeignKey('content_type', 'object_id')

    objects = MessageQueryset().as_manager()

    class Meta:
        ordering = ['-created']
        verbose_name = _("message")
        verbose_name_plural = _("messages")
        permissions = (
            ('view_message', 'View message'),
        )

    def __str__(self):
        return "{}".format(self.subject)

    @property
    def was_readed(self):
        return bool(self.read_at)

    @property
    def is_archived(self):
        return self.folder == self.Folder.ARCHIVED

    @property
    def is_sent(self):
        return self.folder == self.Folder.SENT

    @property
    def is_received(self):
        return self.folder == self.Folder.RECEIVED

    def mark_as_read(self):
        if not self.read_at:
            self.read_at = timezone.now()
            self.save(update_fields=['read_at'])

    def send(self, recipients_ids):
        """ """
        # list to force evaluate the queryset
        send_messages.si(self.pk, list(recipients_ids)).delay()
