from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse_lazy
from django.utils.text import Truncator
from django.shortcuts import get_object_or_404, redirect

from .registers import recipient_register
from .models import Message, MessageContact
from .forms import MessageForm, MessageResponseForm


class ContactNamespaceMixin(object):
    """ """
    contact_namespace = None

    def get_context_data(self, **kwargs):
        context = super(ContactNamespaceMixin, self).get_context_data(**kwargs)
        context.update({
            'contact_namespace': self.get_contact_namespace()
        })
        return context

    def get_contact_namespace(self):
        if not self.contact_namespace:
            raise ValueError("contact namespace not defined")
        return self.contact_namespace


class MessageGroupMixin(object):

    def get_group(self):
        return None

    def dispatch(self, *args, **kwargs):
        self.group_object = self.get_group()
        return super(MessageGroupMixin, self).dispatch(*args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(MessageGroupMixin, self).get_context_data(**kwargs)
        if hasattr(self, 'group_object') and self.group_object:
            context.update({
                'group_object': self.group_object
            })
        return context


class MessageListMixin(ContactNamespaceMixin, MessageGroupMixin):
    """ Defines base settings for lists view """
    model = Message
    folder = Message.Folder.RECEIVED
    paginate_by = 10

    def get_context_data(self, **kwargs):
        context = super(MessageListMixin, self).get_context_data(**kwargs)
        context.update({
            'title': self.title,
            'is_inbox': self.folder == Message.Folder.RECEIVED,
            'is_sentbox': self.folder == Message.Folder.SENT,
            'is_archivedbox': self.folder == Message.Folder.ARCHIVED,
        })
        return context


class MessageReceivedListMixin(MessageListMixin):
    """ """
    title = _("Received")

    def get_queryset(self):
        return Message.objects.inbox(
            self.request.user, self.get_contact_namespace(), self.group_object)


class MessageSentListMixin(MessageListMixin):
    """"""
    folder = Message.Folder.SENT
    title = _("Sent")

    def get_queryset(self):
        return Message.objects.sentbox(
            self.request.user, self.get_contact_namespace(), self.group_object)


class MessageArchivedListMixin(MessageListMixin):
    """ """
    folder = Message.Folder.ARCHIVED
    title = _("Archived")

    def get_queryset(self):
        return Message.objects.archivedbox(
            self.request.user, self.get_contact_namespace(), self.group_object)


class MessageDetailMixin(ContactNamespaceMixin, MessageGroupMixin):
    """ """
    model = Message

    def get(self, *args, **kwargs):
        response = super(MessageDetailMixin, self).get(*args, **kwargs)
        self.object.mark_as_read()
        return response


class MessageFormMixin(object):
    """ Defines form settings """
    model = Message
    form_class = MessageForm

    def get_initial(self):
        initial = super(MessageFormMixin, self).get_initial()
        if self.request.user.is_authenticated():
            initial.update({
                'name': self.request.user.name,
                'email': self.request.user.get_email()
            })
        return initial

    def get_form_kwargs(self):
        """ """
        form_kwargs = super(MessageFormMixin, self).get_form_kwargs()
        form_kwargs.update({
            'user': self.request.user
        })
        return form_kwargs


class MessageCreateMixin(MessageGroupMixin, ContactNamespaceMixin,
                         MessageFormMixin):
    """ """
    has_recipients = True

    def get_recipients(self, form):
        return form.cleaned_data.get('recipients', [])

    def get_recipients_contact(self, form):
        group_pk = self.group_object.pk if self.group_object else None
        namespaces = recipient_register.get_by_namespace(
            self.request.user, self.get_recipients(form), group_pk)
        return MessageContact.objects.get_or_create_contacts(namespaces)

    def get_sender_contact(self, form):
        return MessageContact.objects.get_or_create_contact(
            user=self.request.user,
            namespace=self.get_contact_namespace(),
            name=form.cleaned_data.get('name', None),
            email=form.cleaned_data.get('email', None)
        )

    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.sender = self.get_sender_contact(form)
        if self.group_object:
            self.object.content_object = self.group_object
        self.object.save()
        recipients = self.get_recipients_contact(form)
        self.object.send(recipients.values_list('pk', flat=True))
        return redirect(self.get_success_url())

    def get_form_kwargs(self):
        """ """
        form_kwargs = super(MessageCreateMixin, self).get_form_kwargs()
        if self.has_recipients:
            group_pk = self.group_object.pk if self.group_object else None
            url = reverse_lazy("evotemessages:messages-recipients")
            if group_pk:
                url = "{}?group_pk={}".format(url, group_pk)
            form_kwargs.update({
                'recipients_url': url
            })
        return form_kwargs


class MessageResponseMixin(MessageGroupMixin, ContactNamespaceMixin,
                           MessageFormMixin):
    """ """
    form_class = MessageResponseForm

    def get_form_kwargs(self):
        form_kwargs = super(MessageResponseMixin, self).get_form_kwargs()
        del form_kwargs['instance']
        return form_kwargs

    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.sender_id = self.object.recipient_id
        self.object.content_object = self.object.content_object
        self.object.save()
        self.object.send([self.object.sender_id])
        return redirect(self.get_success_url())

    def get_initial(self):
        initial = super(MessageFormMixin, self).get_initial()
        initial.update({
            'name': self.object.recipient.get_name(),
            'email': self.object.recipient.get_email(),
            'recipients': self.object.sender,
            'subject': Truncator('Re: ' + self.object.subject).chars(253)
        })
        return initial


class ArchiveMessageMixin(object):

    def get(self, request, pk, *args, **kwargs):
        message = get_object_or_404(Message, pk=pk)
        message.folder = Message.Folder.ARCHIVED
        message.save()
        # return redirect(url)
